@@ -2,6 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
+ * Copyright (C) 2017 Osimis, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -39,7 +40,7 @@ static void AddInstance(std::list<std::string>& target,
       !instance.isMember("ID") ||
       instance["ID"].type() != Json::stringValue)
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
   }
   else
   {
@@ -72,7 +73,7 @@ static bool GetSequenceSize(size_t& result,
   {
     OrthancPlugins::Configuration::LogError("The STOW-RS JSON response from DICOMweb server " + server + 
                                             " does not contain the mandatory tag " + upper);
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
   }
   else
   {
@@ -84,7 +85,7 @@ static bool GetSequenceSize(size_t& result,
       (*value) ["Value"].type() != Json::arrayValue)
   {
     OrthancPlugins::Configuration::LogError("Unable to parse STOW-RS JSON response from DICOMweb server " + server);
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
   }
 
   result = (*value) ["Value"].size();
@@ -95,10 +96,12 @@ static bool GetSequenceSize(size_t& result,
 
 static void ParseStowRequest(std::list<std::string>& instances /* out */,
                              std::map<std::string, std::string>& httpHeaders /* out */,
+                             std::map<std::string, std::string>& queryArguments /* out */,
                              const OrthancPluginHttpRequest* request /* in */)
 {
   static const char* RESOURCES = "Resources";
   static const char* HTTP_HEADERS = "HttpHeaders";
+  static const char* QUERY_ARGUMENTS = "Arguments";
 
   OrthancPluginContext* context = OrthancPlugins::Configuration::GetContext();
 
@@ -112,9 +115,10 @@ static void ParseStowRequest(std::list<std::string>& instances /* out */,
     OrthancPlugins::Configuration::LogError("A request to the DICOMweb STOW-RS client must provide a JSON object "
                                             "with the field \"" + std::string(RESOURCES) + 
                                             "\" containing an array of resources to be sent");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
 
+  OrthancPlugins::ParseAssociativeArray(queryArguments, body, QUERY_ARGUMENTS);
   OrthancPlugins::ParseAssociativeArray(httpHeaders, body, HTTP_HEADERS);
 
   Json::Value& resources = body[RESOURCES];
@@ -124,32 +128,32 @@ static void ParseStowRequest(std::list<std::string>& instances /* out */,
   {
     if (resources[i].type() != Json::stringValue)
     {
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
     }
 
     std::string resource = resources[i].asString();
     if (resource.empty())
     {
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_UnknownResource);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_UnknownResource);
     }
 
     // Test whether this resource is an instance
     Json::Value tmp;
-    if (OrthancPlugins::RestApiGetJson(tmp, context, "/instances/" + resource, false))
+    if (OrthancPlugins::RestApiGet(tmp, context, "/instances/" + resource, false))
     {
       AddInstance(instances, tmp);
     }
     // This was not an instance, successively try with series/studies/patients
-    else if ((OrthancPlugins::RestApiGetJson(tmp, context, "/series/" + resource, false) &&
-              OrthancPlugins::RestApiGetJson(tmp, context, "/series/" + resource + "/instances", false)) ||
-             (OrthancPlugins::RestApiGetJson(tmp, context, "/studies/" + resource, false) &&
-              OrthancPlugins::RestApiGetJson(tmp, context, "/studies/" + resource + "/instances", false)) ||
-             (OrthancPlugins::RestApiGetJson(tmp, context, "/patients/" + resource, false) &&
-              OrthancPlugins::RestApiGetJson(tmp, context, "/patients/" + resource + "/instances", false)))
+    else if ((OrthancPlugins::RestApiGet(tmp, context, "/series/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmp, context, "/series/" + resource + "/instances", false)) ||
+             (OrthancPlugins::RestApiGet(tmp, context, "/studies/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmp, context, "/studies/" + resource + "/instances", false)) ||
+             (OrthancPlugins::RestApiGet(tmp, context, "/patients/" + resource, false) &&
+              OrthancPlugins::RestApiGet(tmp, context, "/patients/" + resource + "/instances", false)))
     {
       if (tmp.type() != Json::arrayValue)
       {
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
       }
 
       for (Json::Value::ArrayIndex j = 0; j < tmp.size(); j++)
@@ -159,7 +163,7 @@ static void ParseStowRequest(std::list<std::string>& instances /* out */,
     }
     else
     {
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_UnknownResource);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_UnknownResource);
     }   
   }
 }
@@ -167,6 +171,7 @@ static void ParseStowRequest(std::list<std::string>& instances /* out */,
 
 static void SendStowChunks(const Orthanc::WebServiceParameters& server,
                            const std::map<std::string, std::string>& httpHeaders,
+                           const std::map<std::string, std::string>& queryArguments,
                            const std::string& boundary,
                            Orthanc::ChunkedBuffer& chunks,
                            size_t& countInstances,
@@ -186,8 +191,12 @@ static void SendStowChunks(const Orthanc::WebServiceParameters& server,
 
     OrthancPlugins::MemoryBuffer answerBody(OrthancPlugins::Configuration::GetContext());
     std::map<std::string, std::string> answerHeaders;
+
+    std::string uri;
+    OrthancPlugins::UriEncode(uri, "studies", queryArguments);
+
     OrthancPlugins::CallServer(answerBody, answerHeaders, server, OrthancPluginHttpMethod_Post,
-                               httpHeaders, "studies", body);
+                               httpHeaders, uri, body);
 
     Json::Value response;
     Json::Reader reader;
@@ -200,7 +209,7 @@ static void SendStowChunks(const Orthanc::WebServiceParameters& server,
         !response.isMember("00081199"))
     {
       OrthancPlugins::Configuration::LogError("Unable to parse STOW-RS JSON response from DICOMweb server " + server.GetUrl());
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
     }
 
     size_t size;
@@ -210,7 +219,7 @@ static void SendStowChunks(const Orthanc::WebServiceParameters& server,
       OrthancPlugins::Configuration::LogError("The STOW-RS server was only able to receive " + 
                                               boost::lexical_cast<std::string>(size) + " instances out of " +
                                               boost::lexical_cast<std::string>(countInstances));
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
     }
 
     if (GetSequenceSize(size, response, "00081198", false, server.GetUrl()) &&
@@ -219,7 +228,7 @@ static void SendStowChunks(const Orthanc::WebServiceParameters& server,
       OrthancPlugins::Configuration::LogError("The response from the STOW-RS server contains " + 
                                               boost::lexical_cast<std::string>(size) + 
                                               " items in its Failed SOP Sequence (0008,1198) tag");
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);    
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);    
     }
 
     if (GetSequenceSize(size, response, "0008119A", false, server.GetUrl()) &&
@@ -228,7 +237,7 @@ static void SendStowChunks(const Orthanc::WebServiceParameters& server,
       OrthancPlugins::Configuration::LogError("The response from the STOW-RS server contains " + 
                                               boost::lexical_cast<std::string>(size) + 
                                               " items in its Other Failures Sequence (0008,119A) tag");
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);    
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);    
     }
 
     countInstances = 0;
@@ -244,7 +253,7 @@ void StowClient(OrthancPluginRestOutput* output,
 
   if (request->groupsCount != 1)
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadRequest);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadRequest);
   }
 
   if (request->method != OrthancPluginHttpMethod_Post)
@@ -266,7 +275,7 @@ void StowClient(OrthancPluginRestOutput* output,
     catch (...)
     {
       OrthancPluginFreeString(context, uuid);
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NotEnoughMemory);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_NotEnoughMemory);
     }
 
     OrthancPluginFreeString(context, uuid);
@@ -274,15 +283,16 @@ void StowClient(OrthancPluginRestOutput* output,
 
   std::string mime = "multipart/related; type=application/dicom; boundary=" + boundary;
 
+  std::map<std::string, std::string> queryArguments;
   std::map<std::string, std::string> httpHeaders;
   httpHeaders["Accept"] = "application/json";
   httpHeaders["Expect"] = "";
   httpHeaders["Content-Type"] = mime;
 
   std::list<std::string> instances;
-  ParseStowRequest(instances, httpHeaders, request);
+  ParseStowRequest(instances, httpHeaders, queryArguments, request);
 
-  OrthancPlugins::Configuration::LogInfo("Sending " + boost::lexical_cast<std::string>(instances.size()) + 
+  OrthancPlugins::Configuration::LogInfo("Sending " + boost::lexical_cast<std::string>(instances.size()) +
                                          " instances using STOW-RS to DICOMweb server: " + server.GetUrl());
 
   Orthanc::ChunkedBuffer chunks;
@@ -300,11 +310,11 @@ void StowClient(OrthancPluginRestOutput* output,
       chunks.AddChunk(dicom.GetData(), dicom.GetSize());
       countInstances ++;
 
-      SendStowChunks(server, httpHeaders, boundary, chunks, countInstances, false);
+      SendStowChunks(server, httpHeaders, queryArguments, boundary, chunks, countInstances, false);
     }
   }
 
-  SendStowChunks(server, httpHeaders, boundary, chunks, countInstances, true);
+  SendStowChunks(server, httpHeaders, queryArguments, boundary, chunks, countInstances, true);
 
   std::string answer = "{}\n";
   OrthancPluginAnswerBuffer(context, output, answer.c_str(), answer.size(), "application/json");
@@ -317,7 +327,7 @@ static bool GetStringValue(std::string& target,
 {
   if (json.type() != Json::objectValue)
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
   else if (!json.isMember(key))
   {
@@ -327,7 +337,7 @@ static bool GetStringValue(std::string& target,
   else if (json[key].type() != Json::stringValue)
   {
     OrthancPlugins::Configuration::LogError("The field \"" + key + "\" in a JSON object should be a string");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
   else
   {
@@ -362,9 +372,9 @@ void GetFromServer(OrthancPluginRestOutput* output,
       body.type() != Json::objectValue ||
       !GetStringValue(tmp, body, URI))
   {
-    OrthancPlugins::Configuration::LogError("A request to the DICOMweb STOW-RS client must provide a JSON object "
+    OrthancPlugins::Configuration::LogError("A request to the DICOMweb client must provide a JSON object "
                                             "with the field \"Uri\" containing the URI of interest");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
 
   std::map<std::string, std::string> getArguments;
@@ -412,6 +422,7 @@ void GetFromServer(OrthancPluginRestOutput* output,
 static void RetrieveFromServerInternal(std::set<std::string>& instances,
                                        const Orthanc::WebServiceParameters& server,
                                        const std::map<std::string, std::string>& httpHeaders,
+                                       const std::map<std::string, std::string>& getArguments,
                                        const Json::Value& resource)
 {
   static const std::string STUDY = "Study";
@@ -426,7 +437,7 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
   {
     OrthancPlugins::Configuration::LogError("Resources of interest for the DICOMweb WADO-RS Retrieve client "
                                             "must be provided as a JSON object");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
 
   std::string study, series, instance;
@@ -435,7 +446,7 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
   {
     OrthancPlugins::Configuration::LogError("A non-empty \"" + STUDY + "\" field is mandatory for the "
                                             "DICOMweb WADO-RS Retrieve client");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
 
   GetStringValue(series, resource, SERIES);
@@ -446,19 +457,22 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
   {
     OrthancPlugins::Configuration::LogError("When specifying a \"" + INSTANCE + "\" field in a call to DICOMweb "
                                             "WADO-RS Retrieve client, the \"" + SERIES + "\" field is mandatory");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
 
-  std::string uri = "studies/" + study;
+  std::string tmpUri = "studies/" + study;
   if (!series.empty())
   {
-    uri += "/series/" + series;
+    tmpUri += "/series/" + series;
     if (!instance.empty())
     {
-      uri += "/instances/" + instance;
+      tmpUri += "/instances/" + instance;
     }
   }
 
+  std::string uri;
+  OrthancPlugins::UriEncode(uri, tmpUri, getArguments);
+
   OrthancPlugins::MemoryBuffer answerBody(context);
   std::map<std::string, std::string> answerHeaders;
   OrthancPlugins::CallServer(answerBody, answerHeaders, server, OrthancPluginHttpMethod_Get, httpHeaders, uri, "");
@@ -479,7 +493,7 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
   if (contentType.empty())
   {
     OrthancPlugins::Configuration::LogError("No Content-Type provided by the remote WADO-RS server");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
   }
 
   Orthanc::Toolbox::ToLowerCase(contentType[0]);
@@ -487,7 +501,7 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
   {
     OrthancPlugins::Configuration::LogError("The remote WADO-RS server answers with a \"" + contentType[0] +
                                             "\" Content-Type, but \"" + MULTIPART_RELATED + "\" is expected");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
   }
 
   std::string type, boundary;
@@ -504,6 +518,18 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
       if (s == "type")
       {
         type = Orthanc::Toolbox::StripSpaces(tokens[1]);
+
+        // This covers the case where the content-type is quoted,
+        // which COULD be the case 
+        // cf. https://tools.ietf.org/html/rfc7231#section-3.1.1.1
+        size_t len = type.length();
+        if (len >= 2 &&
+            type[0] == '"' &&
+            type[len - 1] == '"')
+        {
+          type = type.substr(1, len - 2);
+        }
+        
         Orthanc::Toolbox::ToLowerCase(type);
       }
       else if (s == "boundary")
@@ -517,13 +543,13 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
   {
     OrthancPlugins::Configuration::LogError("The remote WADO-RS server answers with a \"" + type +
                                             "\" multipart Content-Type, but \"" + APPLICATION_DICOM + "\" is expected");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
   }
 
   if (boundary.empty())
   {
     OrthancPlugins::Configuration::LogError("The remote WADO-RS server does not provide a boundary for its multipart answer");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
   }
 
   std::vector<OrthancPlugins::MultipartItem> parts;
@@ -540,7 +566,7 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
     if (parts[i].contentType_ != APPLICATION_DICOM)
     {
       OrthancPlugins::Configuration::LogError("The remote WADO-RS server has provided a non-DICOM file in its multipart answer");
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);      
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);      
     }
 
     OrthancPlugins::MemoryBuffer tmp(context);
@@ -553,7 +579,7 @@ static void RetrieveFromServerInternal(std::set<std::string>& instances,
         !result.isMember("ID") ||
         result["ID"].type() != Json::stringValue)
     {
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);      
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);      
     }
     else
     {
@@ -570,6 +596,7 @@ void RetrieveFromServer(OrthancPluginRestOutput* output,
 {
   static const std::string RESOURCES("Resources");
   static const char* HTTP_HEADERS = "HttpHeaders";
+  static const std::string GET_ARGUMENTS = "Arguments";
 
   OrthancPluginContext* context = OrthancPlugins::Configuration::GetContext();
 
@@ -590,16 +617,20 @@ void RetrieveFromServer(OrthancPluginRestOutput* output,
   {
     OrthancPlugins::Configuration::LogError("A request to the DICOMweb WADO-RS Retrieve client must provide a JSON object "
                                             "with the field \"" + RESOURCES + "\" containing an array of resources");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
   }
 
   std::map<std::string, std::string> httpHeaders;
   OrthancPlugins::ParseAssociativeArray(httpHeaders, body, HTTP_HEADERS);
 
+  std::map<std::string, std::string> getArguments;
+  OrthancPlugins::ParseAssociativeArray(getArguments, body, GET_ARGUMENTS);
+
+
   std::set<std::string> instances;
   for (Json::Value::ArrayIndex i = 0; i < body[RESOURCES].size(); i++)
   {
-    RetrieveFromServerInternal(instances, server, httpHeaders, body[RESOURCES][i]);
+    RetrieveFromServerInternal(instances, server, httpHeaders, getArguments, body[RESOURCES][i]);
   }
 
   Json::Value status = Json::objectValue;
