@@ -2,6 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
+ * Copyright (C) 2017 Osimis, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -153,6 +154,12 @@ void StowCallback(OrthancPluginRestOutput* output,
   std::vector<OrthancPlugins::MultipartItem> items;
   OrthancPlugins::ParseMultipartBody(items, context, request->body, request->bodySize, boundary);
 
+  for (size_t i = 0; i < items.size(); i++)
+  {
+    OrthancPlugins::Configuration::LogInfo("Detected multipart item with content type \"" + 
+                                           items[i].contentType_ + "\" of size " + 
+                                           boost::lexical_cast<std::string>(items[i].size_));
+  }  
 
   for (size_t i = 0; i < items.size(); i++)
   {
