@@ -2,6 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
+ * Copyright (C) 2017 Osimis, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -174,7 +175,7 @@ static void AnswerListOfDicomInstances(OrthancPluginRestOutput* output,
   OrthancPluginContext* context = OrthancPlugins::Configuration::GetContext();
 
   Json::Value instances;
-  if (!OrthancPlugins::RestApiGetJson(instances, context, resource + "/instances", false))
+  if (!OrthancPlugins::RestApiGet(instances, context, resource + "/instances", false))
   {
     // Internal error
     OrthancPluginSendHttpStatusCode(context, output, 400);
@@ -183,7 +184,7 @@ static void AnswerListOfDicomInstances(OrthancPluginRestOutput* output,
 
   if (OrthancPluginStartMultipartAnswer(context, output, "related", "application/dicom"))
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
   }
   
   for (Json::Value::ArrayIndex i = 0; i < instances.size(); i++)
@@ -194,7 +195,7 @@ static void AnswerListOfDicomInstances(OrthancPluginRestOutput* output,
     if (dicom.RestApiGet(uri, false) &&
         OrthancPluginSendMultipartItem(context, output, dicom.GetData(), dicom.GetSize()) != 0)
     {
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
     }
   }
 }
@@ -217,7 +218,7 @@ static void AnswerMetadata(OrthancPluginRestOutput* output,
   else
   {
     Json::Value instances;
-    if (!OrthancPlugins::RestApiGetJson(instances, context, resource + "/instances", false))
+    if (!OrthancPlugins::RestApiGet(instances, context, resource + "/instances", false))
     {
       // Internal error
       OrthancPluginSendHttpStatusCode(context, output, 400);
@@ -310,7 +311,7 @@ static bool LocateSeries(OrthancPluginRestOutput* output,
   }
   
   Json::Value study;
-  if (!OrthancPlugins::RestApiGetJson(study, context, "/series/" + id + "/study", false))
+  if (!OrthancPlugins::RestApiGet(study, context, "/series/" + id + "/study", false))
   {
     OrthancPluginSendHttpStatusCode(context, output, 404);
     return false;
@@ -358,8 +359,8 @@ bool LocateInstance(OrthancPluginRestOutput* output,
   }
   
   Json::Value study, series;
-  if (!OrthancPlugins::RestApiGetJson(series, context, "/instances/" + id + "/series", false) ||
-      !OrthancPlugins::RestApiGetJson(study, context, "/instances/" + id + "/study", false))
+  if (!OrthancPlugins::RestApiGet(series, context, "/instances/" + id + "/series", false) ||
+      !OrthancPlugins::RestApiGet(study, context, "/instances/" + id + "/study", false))
   {
     OrthancPluginSendHttpStatusCode(context, output, 404);
     return false;
@@ -436,14 +437,14 @@ void RetrieveDicomInstance(OrthancPluginRestOutput* output,
     {
       if (OrthancPluginStartMultipartAnswer(context, output, "related", "application/dicom"))
       {
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
       }
 
       OrthancPlugins::MemoryBuffer dicom(context);
       if (dicom.RestApiGet(uri + "/file", false) &&
           OrthancPluginSendMultipartItem(context, output, dicom.GetData(), dicom.GetSize()) != 0)
       {
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);
       }
     }
   }
@@ -612,7 +613,7 @@ void RetrieveBulkData(OrthancPluginRestOutput* output,
       if (OrthancPluginStartMultipartAnswer(context, output, "related", "application/octet-stream") != 0 ||
           OrthancPluginSendMultipartItem(context, output, result.c_str(), result.size()) != 0)
       {
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_Plugin);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_Plugin);
       }
     }
     else
