@@ -2,6 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
+ * Copyright (C) 2017 Osimis, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -68,7 +69,7 @@ static gdcm::TransferSyntax ParseTransferSyntax(const OrthancPluginHttpRequest*
 
       if (tokens[0] != "multipart/related")
       {
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_ParameterOutOfRange);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
       }
 
       std::string type("application/octet-stream");
@@ -81,7 +82,7 @@ static gdcm::TransferSyntax ParseTransferSyntax(const OrthancPluginHttpRequest*
 
         if (parsed.size() != 2)
         {
-          throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadRequest);
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_BadRequest);
         }
 
         if (parsed[0] == "type")
@@ -105,12 +106,70 @@ static gdcm::TransferSyntax ParseTransferSyntax(const OrthancPluginHttpRequest*
         {
           OrthancPlugins::Configuration::LogError("DICOMweb RetrieveFrames: Cannot specify a transfer syntax (" + 
                                                   transferSyntax + ") for default Little Endian uncompressed pixel data");
-          throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadRequest);
+          throw Orthanc::OrthancException(Orthanc::ErrorCode_BadRequest);
         }
       }
       else
       {
-        // http://dicom.nema.org/medical/dicom/current/output/html/part18.html#table_6.5-1
+        /**
+         * DICOM 2017c
+         * http://dicom.nema.org/medical/dicom/current/output/html/part18.html#table_6.1.1.8-3b
+         **/
+        if (type == "image/jpeg" && (transferSyntax.empty() ||  // Default
+                                     transferSyntax == "1.2.840.10008.1.2.4.70"))
+        {
+          return gdcm::TransferSyntax::JPEGLosslessProcess14_1;
+        }
+        else if (type == "image/jpeg" && transferSyntax == "1.2.840.10008.1.2.4.50")
+        {
+          return gdcm::TransferSyntax::JPEGBaselineProcess1;
+        }
+        else if (type == "image/jpeg" && transferSyntax == "1.2.840.10008.1.2.4.51")
+        {
+          return gdcm::TransferSyntax::JPEGExtendedProcess2_4;
+        }
+        else if (type == "image/jpeg" && transferSyntax == "1.2.840.10008.1.2.4.57")
+        {
+          return gdcm::TransferSyntax::JPEGLosslessProcess14;
+        }
+        else if (type == "image/x-dicom-rle" && (transferSyntax.empty() ||  // Default
+                                                 transferSyntax == "1.2.840.10008.1.2.5"))
+        {
+          return gdcm::TransferSyntax::RLELossless;
+        }
+        else if (type == "image/x-jls" && (transferSyntax.empty() ||  // Default
+                                           transferSyntax == "1.2.840.10008.1.2.4.80"))
+        {
+          return gdcm::TransferSyntax::JPEGLSLossless;
+        }
+        else if (type == "image/x-jls" && transferSyntax == "1.2.840.10008.1.2.4.81")
+        {
+          return gdcm::TransferSyntax::JPEGLSNearLossless;
+        }
+        else if (type == "image/jp2" && (transferSyntax.empty() ||  // Default
+                                         transferSyntax == "1.2.840.10008.1.2.4.90"))
+        {
+          return gdcm::TransferSyntax::JPEG2000Lossless;
+        }
+        else if (type == "image/jp2" && transferSyntax == "1.2.840.10008.1.2.4.91")
+        {
+          return gdcm::TransferSyntax::JPEG2000;
+        }
+        else if (type == "image/jpx" && (transferSyntax.empty() ||  // Default
+                                         transferSyntax == "1.2.840.10008.1.2.4.92"))
+        {
+          return gdcm::TransferSyntax::JPEG2000Part2Lossless;
+        }
+        else if (type == "image/jpx" && transferSyntax == "1.2.840.10008.1.2.4.93")
+        {
+          return gdcm::TransferSyntax::JPEG2000Part2;
+        }
+
+
+        /**
+         * Backward compatibility with DICOM 2014a
+         * http://dicom.nema.org/medical/dicom/2014a/output/html/part18.html#table_6.5-1
+         **/
         if (type == "image/dicom+jpeg" && transferSyntax == "1.2.840.10008.1.2.4.50")
         {
           return gdcm::TransferSyntax::JPEGBaselineProcess1;
@@ -160,12 +219,11 @@ static gdcm::TransferSyntax ParseTransferSyntax(const OrthancPluginHttpRequest*
         {
           return gdcm::TransferSyntax::JPEG2000Part2;
         }
-        else
-        {
-          OrthancPlugins::Configuration::LogError("DICOMweb RetrieveFrames: Transfer syntax \"" + 
+
+
+        OrthancPlugins::Configuration::LogError("DICOMweb RetrieveFrames: Transfer syntax \"" + 
                                                   transferSyntax + "\" is incompatible with media type \"" + type + "\"");
-          throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadRequest);
-        }
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_BadRequest);
       }
     }
   }
@@ -199,7 +257,7 @@ static void ParseFrameList(std::list<unsigned int>& frames,
     if (frame <= 0)
     {
       OrthancPlugins::Configuration::LogError("Invalid frame number (must be > 0): " + tokens[i]);
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_ParameterOutOfRange);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
     }
 
     frames.push_back(static_cast<unsigned int>(frame - 1));
@@ -210,46 +268,48 @@ static void ParseFrameList(std::list<unsigned int>& frames,
 
 static const char* GetMimeType(const gdcm::TransferSyntax& syntax)
 {
+  // http://dicom.nema.org/medical/dicom/current/output/html/part18.html#table_6.1.1.8-3b
+  
   switch (syntax)
   {
     case gdcm::TransferSyntax::ImplicitVRLittleEndian:
       return "application/octet-stream";
 
     case gdcm::TransferSyntax::JPEGBaselineProcess1:
-      return "image/dicom+jpeg; transfer-syntax=1.2.840.10008.1.2.4.50";
+      return "image/jpeg; transfer-syntax=1.2.840.10008.1.2.4.50";
 
     case gdcm::TransferSyntax::JPEGExtendedProcess2_4:
-      return "image/dicom+jpeg; transfer-syntax=1.2.840.10008.1.2.4.51";
+      return "image/jpeg; transfer-syntax=1.2.840.10008.1.2.4.51";
     
     case gdcm::TransferSyntax::JPEGLosslessProcess14:
-      return "image/dicom+jpeg; transfer-syntax=1.2.840.10008.1.2.4.57";
+      return "image/jpeg; transfer-syntax=1.2.840.10008.1.2.4.57";
 
     case gdcm::TransferSyntax::JPEGLosslessProcess14_1:
-      return "image/dicom+jpeg; transferSyntax=1.2.840.10008.1.2.4.70";
+      return "image/jpeg; transferSyntax=1.2.840.10008.1.2.4.70";
     
     case gdcm::TransferSyntax::RLELossless:
-      return "image/dicom+rle; transferSyntax=1.2.840.10008.1.2.5";
+      return "image/x-dicom-rle; transferSyntax=1.2.840.10008.1.2.5";
 
     case gdcm::TransferSyntax::JPEGLSLossless:
-      return "image/dicom+jpeg-ls; transferSyntax=1.2.840.10008.1.2.4.80";
+      return "image/x-jls; transferSyntax=1.2.840.10008.1.2.4.80";
 
     case gdcm::TransferSyntax::JPEGLSNearLossless:
-      return "image/dicom+jpeg-ls; transfer-syntax=1.2.840.10008.1.2.4.81";
+      return "image/x-jls; transfer-syntax=1.2.840.10008.1.2.4.81";
 
     case gdcm::TransferSyntax::JPEG2000Lossless:
-      return "image/dicom+jp2; transferSyntax=1.2.840.10008.1.2.4.90";
+      return "image/jp2; transferSyntax=1.2.840.10008.1.2.4.90";
 
     case gdcm::TransferSyntax::JPEG2000:
-      return "image/dicom+jp2; transfer-syntax=1.2.840.10008.1.2.4.91";
+      return "image/jp2; transfer-syntax=1.2.840.10008.1.2.4.91";
 
     case gdcm::TransferSyntax::JPEG2000Part2Lossless:
-      return "image/dicom+jpx; transferSyntax=1.2.840.10008.1.2.4.92";
+      return "image/jpx; transferSyntax=1.2.840.10008.1.2.4.92";
 
     case gdcm::TransferSyntax::JPEG2000Part2:
-      return "image/dicom+jpx; transfer-syntax=1.2.840.10008.1.2.4.93";
+      return "image/jpx; transfer-syntax=1.2.840.10008.1.2.4.93";
 
     default:
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
   }
 }
 
@@ -275,7 +335,7 @@ static void AnswerSingleFrame(OrthancPluginRestOutput* output,
 
   if (error != OrthancPluginErrorCode_Success)
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NetworkProtocol);      
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_NetworkProtocol);      
   }
 }
 
@@ -289,7 +349,7 @@ static bool AnswerFrames(OrthancPluginRestOutput* output,
 {
   if (!dicom.GetDataSet().FindDataElement(OrthancPlugins::DICOM_TAG_PIXEL_DATA))
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_IncompatibleImageFormat);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_IncompatibleImageFormat);
   }
 
   const gdcm::DataElement& pixelData = dicom.GetDataSet().GetDataElement(OrthancPlugins::DICOM_TAG_PIXEL_DATA);
@@ -308,23 +368,24 @@ static bool AnswerFrames(OrthancPluginRestOutput* output,
     if (pixelData.GetByteValue() == NULL)
     {
       OrthancPlugins::Configuration::LogError("Image was not properly decoded");
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);      
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);      
     }
 
-    int width, height, bits;
+    int width, height, bits, samplesPerPixel;
 
     if (!dicom.GetIntegerTag(height, *dictionary_, OrthancPlugins::DICOM_TAG_ROWS) ||
         !dicom.GetIntegerTag(width, *dictionary_, OrthancPlugins::DICOM_TAG_COLUMNS) ||
-        !dicom.GetIntegerTag(bits, *dictionary_, OrthancPlugins::DICOM_TAG_BITS_ALLOCATED))
+        !dicom.GetIntegerTag(bits, *dictionary_, OrthancPlugins::DICOM_TAG_BITS_ALLOCATED) || 
+        !dicom.GetIntegerTag(samplesPerPixel, *dictionary_, OrthancPlugins::DICOM_TAG_SAMPLES_PER_PIXEL))
     {
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
     }
 
-    size_t frameSize = height * width * bits / 8;
+    size_t frameSize = height * width * bits * samplesPerPixel / 8;
     
     if (pixelData.GetByteValue()->GetLength() % frameSize != 0)
     {
-      throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);      
+      throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);      
     }
 
     size_t framesCount = pixelData.GetByteValue()->GetLength() / frameSize;
@@ -348,7 +409,7 @@ static bool AnswerFrames(OrthancPluginRestOutput* output,
       {
         OrthancPlugins::Configuration::LogError("Trying to access frame number " + boost::lexical_cast<std::string>(*frame + 1) + 
                                                 " of an image with " + boost::lexical_cast<std::string>(framesCount) + " frames");
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_ParameterOutOfRange);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
       }
       else
       {
@@ -381,7 +442,7 @@ static bool AnswerFrames(OrthancPluginRestOutput* output,
                                                 " of an image with " + 
                                                 boost::lexical_cast<std::string>(fragments->GetNumberOfFragments()) + 
                                                 " frames");
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_ParameterOutOfRange);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_ParameterOutOfRange);
       }
       else
       {
@@ -413,7 +474,7 @@ void RetrieveFrames(OrthancPluginRestOutput* output,
   OrthancPlugins::MemoryBuffer content(context);
   if (LocateInstance(output, uri, request) &&
       content.RestApiGet(uri + "/file", false) &&
-      OrthancPlugins::RestApiGetJson(header, context, uri + "/header?simplify", false))
+      OrthancPlugins::RestApiGet(header, context, uri + "/header?simplify", false))
   {
     {
       std::string s = "DICOMweb RetrieveFrames on " + uri + ", frames: ";
@@ -476,14 +537,14 @@ void RetrieveFrames(OrthancPluginRestOutput* output,
       if (!reader.Read())
       {
         OrthancPlugins::Configuration::LogError("Cannot decode the image");
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadFileFormat);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_BadFileFormat);
       }
 
       change.SetInput(reader.GetImage());
       if (!change.Change())
       {
         OrthancPlugins::Configuration::LogError("Cannot change the transfer syntax of the image");
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_InternalError);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_InternalError);
       }
 
       gdcm::ImageWriter writer;
@@ -494,7 +555,7 @@ void RetrieveFrames(OrthancPluginRestOutput* output,
       writer.SetStream(ss);
       if (!writer.Write())
       {
-        throw OrthancPlugins::PluginException(OrthancPluginErrorCode_NotEnoughMemory);
+        throw Orthanc::OrthancException(Orthanc::ErrorCode_NotEnoughMemory);
       }
 
       OrthancPlugins::ParsedDicomFile transcoded(ss.str());
