@@ -2,6 +2,7 @@
  * Orthanc - A Lightweight, RESTful DICOM Store
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
+ * Copyright (C) 2017 Osimis, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Affero General Public License
@@ -116,7 +117,7 @@ static bool LocateInstance(std::string& instance,
     else
     {
       Json::Value info;
-      if (!OrthancPlugins::RestApiGetJson(info, context, "/instances/" + instance + "/series", false) ||
+      if (!OrthancPlugins::RestApiGet(info, context, "/instances/" + instance + "/series", false) ||
           info["MainDicomTags"]["SeriesInstanceUID"] != seriesUid)
       {
         OrthancPlugins::Configuration::LogError("WADO-URI: Instance " + objectUid + " does not belong to series " + seriesUid);
@@ -136,7 +137,7 @@ static bool LocateInstance(std::string& instance,
     else
     {
       Json::Value info;
-      if (!OrthancPlugins::RestApiGetJson(info, context, "/instances/" + instance + "/study", false) ||
+      if (!OrthancPlugins::RestApiGet(info, context, "/instances/" + instance + "/study", false) ||
           info["MainDicomTags"]["StudyInstanceUID"] != studyUid)
       {
         OrthancPlugins::Configuration::LogError("WADO-URI: Instance " + objectUid + " does not belong to study " + studyUid);
@@ -165,7 +166,7 @@ static void AnswerDicom(OrthancPluginRestOutput* output,
   else
   {
     OrthancPlugins::Configuration::LogError("WADO-URI: Unable to retrieve DICOM file from " + uri);
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_Plugin);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_Plugin);
   }
 }
 
@@ -200,7 +201,7 @@ static void AnswerPngPreview(OrthancPluginRestOutput* output,
   }
   else
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_Plugin);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_Plugin);
   }
 }
 
@@ -214,7 +215,7 @@ static void AnswerJpegPreview(OrthancPluginRestOutput* output,
   OrthancPlugins::MemoryBuffer png(context);
   if (!RetrievePngPreview(png, instance))
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_Plugin);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_Plugin);
   }
   
   OrthancPlugins::OrthancImage image(context);
@@ -237,7 +238,7 @@ void WadoUriCallback(OrthancPluginRestOutput* output,
   std::string contentType = "image/jpg";  // By default, JPEG image will be returned
   if (!LocateInstance(instance, contentType, request))
   {
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_UnknownResource);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_UnknownResource);
   }
 
   if (contentType == "application/dicom")
@@ -256,6 +257,6 @@ void WadoUriCallback(OrthancPluginRestOutput* output,
   else
   {
     OrthancPlugins::Configuration::LogError("WADO-URI: Unsupported content type: \"" + contentType + "\"");
-    throw OrthancPlugins::PluginException(OrthancPluginErrorCode_BadRequest);
+    throw Orthanc::OrthancException(Orthanc::ErrorCode_BadRequest);
   }
 }
